% Tortuous cardiac intercalated discs modulate ephaptic coupling
% Ena Ivanovic and Jan P. Kucera
% 2022
% Department of Physiology, University of Bern, Bern, Switzerland 

% Use under the terms of the CC-BY license (Creative Commons Attribuition 4.0 International)

close all
clear all

% Parameters to be set by the user ----------------------------------------

% Space units in cm
% Time is in ms

% Choose mesh folding pattern 
Orientation = 'Concentric'; % 'Radial' % 'Reference'
Amplitude = 4; % 2 % 4 % 8 % 16
Number = 4; % 2 % 4 % 8 % 0
filename = [Orientation 'A' num2str(Amplitude) 'N' num2str(Number) '.mat'];

% Choose Na+ channel distribution 
Config = 'cluster'; % 'uniform'  

% Variable parameters
CleftWidth = 0.03*0.0001; % in cm; Cleft width 30 nm
gapFactor=0;  % level of gap junctional coupling 0% of normal value 2.53 uS (Shaw & Rudy 1997) 

% -------------------------------------------------------------------------

%% Load mesh

load(filename,'pc','tc','pclust')  

% Converted Comsol values to cm, to consistently work in cm
p=0.001*pc; % 
pclust=0.001*pclust; % 
t=tc; 

% Rotate mesh in x,y axes
p=[p(:,2) p(:,3) p(:,1)];
pclust=[pclust(:,2) pclust(:,3) pclust(:,1)];

[ntri,~]=size(t);
[npoints,~]=size(p);

% Display ID
figure()
trimesh(t,p(:,1),p(:,2),p(:,3),'EdgeColor','k','FaceColor','[0.7 0.7 0.7]');
axis equal 

%% Set parameters

% Constant parameter
DiscCenterX = 0;
DiscCenterY = 0;
DiscRadius = 11*0.0001;
Lcell = 100*0.0001; 
verysmall=0.1*DiscRadius;


SimDur=3;
dt=0.00005; % (ms) 50 ns
frec=100;  % downsampling

Cm=1; % specific membrane capacitance
SigmaExt=1000*(1/150); % 150 Ohm.cm -> mS/cm
specific_gj_resistance=1.5; % Ohm.cm2 (Luo and Rudy, 1994)

gNamax=23; % maximal conductance (mS/cm2) 
gKmax=0.3; % maximal conductance (mS/cm2)

ENa=55; % Nernst potential Na (mV)
EK=-85;  % Nernst potential K (mV)
Vrest=EK; % Resting potential (mV)

% Calculate areas
Alat=2*pi*DiscRadius*Lcell; % Area lateral membrane (cm2) 
Ajun= pi*(DiscRadius^2); % Area flat ID membrane (cm2) 
Acyl1=Alat+Ajun; % Cell 1 (cm2) 
Acyl2=Alat+Ajun; % Cell 2 (cm2) 
Aflat=Alat+2*Ajun; % Area cell membrane with only flat IDs (cm2) 

[~,~,~,Ac,~]=f_kmf_lintric_inh_3D(p',t',1,1,1);
AID = sum(Ac); % Area (tortuous) ID membrane (cm2) 
SF_ID =Ajun/AID; % Tortuosity factor (unitless)
Atort = Alat + Ajun + AID; % Area cell membrane with tortuous ID (cm2) 

% Current clamp protocol
Istim1 = -150 * Aflat; % (uA)
Istim2 = 0; % (uA)
t_on=0.05; % (ms)
t_off=t_on+0.5; % (ms)


% Parameters of the non cleft membranes
Cap1=Acyl1*Cm; % uF
Cap2=Acyl2*Cm; % uF

% Distribition Na+ channels in cell membrane (Hichri et al. 2018; Ivanovic & Kucera 2021)
INafactor=5.05; % (unitless)
INafactorNoncleft1=0.555; % (unitless)
INafactorNoncleft2=0.555; % (unitless)

% Gap junctional coupling
specific_gj_conductance=1/specific_gj_resistance; % (S/cm2)
G=1000*gapFactor*(Ajun/2)*specific_gj_conductance; % Total intercellular conductance (mS)

% Find triangles for Na+ channel cluster
switch (Config)
    
    case 'cluster'

boole=zeros(ntri,3);
for i=1:ntri
   for j=1:3 % 3 corners per triangle
        ix=t(i,j);
        b=ismember(p(ix,:),pclust,'rows');              
        boole(i,j)=b;
   end
end
% Indices of boundary triangles that are on the inner surface of pe domain
ixc=find(sum(boole,2)==3);
ttclust=t(ixc,:); % inner surface triangles cell2
figure()
trimesh(t,p(:,1),p(:,2),p(:,3),'facecolor','[0.7 0.7 0.7]','edgecolor','k');
hold on
trimesh(ttclust,p(:,1),p(:,2),p(:,3),'facecolor','r','edgecolor','k');
hold off
axis equal

[~,~,~,AcNa,~]=f_kmf_lintric_inh_3D(p',ttclust',1,1,1);
ANa = sum (AcNa); 
ClusterFactorNa =AID/ANa; 

end

%% -------------------------------------------------------------------------

kk=SigmaExt.*CleftWidth; % cm * mS/cm -> mS; conductance of a "square" of extracellular space
mm=Cm; % (uF/cm2)
ff=1; % (unitless)

% Scaling conductance TRIANGLEWISE, Computation of V, Iion and gates NODEWISE
[K,M,F,A,X]=f_kmf_lintric_inh_3D(p',t',kk,mm,ff); % kk, mm and ff are either triangle-wise or scalar
% kk will always remain scalar unless one incorporates heterogeneities in cleft width or extracellular resistance
% K is in mS, laplacian operator incorporating, if needed, heterogeneities of clw as different entries in kk
% M is in uF, capacitive mass matrix
% F is in area units (since ff=1)
% A is the area (cm^2) of the triangles
% Here, K and F are the most important, the others will not be used

% Since we assume that Cm is homogeneous and the same for both membranes;
% no need to differentiate between M for the two cells

%% Ionic currents parameters

%========================= cell1 : Driver cell ============================
IK_scaling_factor1=ones(ntri,1); % unitless

switch (Config)
       case 'uniform'
INa_scaling_factor1=ones(ntri,1).*INafactor; % unitless
       case 'cluster'
INa_scaling_factor1=zeros(ntri,1); % unitless
INa_scaling_factor1(ixc)=INafactor*ClusterFactorNa;
end

% Compute scaled load vector for INa, will be important if gNa is not homogeneous
[~,~,Fina1,~,~]=f_kmf_lintric_inh_3D(p',t',1,INa_scaling_factor1,INa_scaling_factor1); % compute scaled load vector for INa in cell 1
[~,~,Fik1,~,~]=f_kmf_lintric_inh_3D(p',t',1,IK_scaling_factor1,IK_scaling_factor1); % compute scaled load vector for INa in cell 1
% Fina1 and Fik1: unit: area, since INa_scaling_factor1 is unitless (normalized)

Fina1=Fina1.*SF_ID; 
Fik1=Fik1.*SF_ID; 

gNa1=sum(gNamax.*Fina1); 
gK1=sum(gKmax.*Fik1); 


%========================= cell2 : Follower cell ==========================
IK_scaling_factor2=ones(ntri,1); % unitless

switch (Config)
       case 'uniform'
INa_scaling_factor2=ones(ntri,1).*INafactor; % unitless
       case 'cluster'
INa_scaling_factor2=zeros(ntri,1); % unitless
INa_scaling_factor2(ixc)=INafactor*ClusterFactorNa;
end

% Compute scaled load vector for INa, will be important if gNa is not homogeneous
[~,~,Fina2,~,~]=f_kmf_lintric_inh_3D(p',t',1,INa_scaling_factor2,INa_scaling_factor2); % compute scaled load vector for INa in cell 1
[~,~,Fik2,~,~]=f_kmf_lintric_inh_3D(p',t',1,IK_scaling_factor2,IK_scaling_factor2); % compute scaled load vector for INa in cell 1
% Fina2 and Fik2: unit: area, since INa_scaling_factor2 is unitless (normalized)

Fina2=Fina2.*SF_ID; 
Fik2=Fik2.*SF_ID; 

gNa2=sum(gNamax.*Fina2); 
gK2=sum(gKmax.*Fik2); 

%--------------------------------------------------------------------------

%Calculate vector m columnwise as sum of the rows of M 
mvector=sum(M,2); % parse
mscalar=full(sum(mvector)); % Calculate scalar mscalar as sum of mvector

% Assemble matrices for the double current clamp situation
K=[ [K sparse(npoints,2)]; [sparse(2, npoints) G*sparse([[1 -1];[-1 1];]) ];];
M=[ [2*M -mvector -mvector]; [-mvector' Cap1+mscalar 0]; [-mvector' 0 Cap2+mscalar];];
% Both matrices are now sparse npoints+2 x npoints+2 !


%% Boundary conditions

dFromCenter = (p(:,1)-DiscCenterX).^2 + (p(:,2)-DiscCenterY).^2;
dc = ( abs(dFromCenter-(DiscRadius^2))<verysmall^2 ); 
iNeuDiscOnly=find(dc~=1); % indices of points with non Dirichlet condition (Neumann)

dc=[dc; false; false]; % Add 2 logical values for the 2 intracellular points
iDir=find(dc==1); % indices of points with Dirichlet condition
iNeu=find(dc~=1); % indices of points with non Dirichlet condition (Neumann)
nDir=length(iDir);
nNeu=length(iNeu);
vDir=zeros(nDir,1); % vector of potentials at the dc points, here all 0

% Display boundary conditions
figure() 
trimesh(t,p(:,1),p(:,2),p(:,3),'EdgeColor','k','FaceColor','none');
hold on 
plot3(p(iDir,1),p(iDir,2),p(iDir,3),'xb')
hold off 
axis equal

%% Simulation

time=0; % (ms)

% Split as
% [Kd  Kx][Vd]=[Md  Mx][dVd/dt]+[iIonD]
% [Kxt Kn][Vn]=[Mxt Mn][dVn/dt]+[iIonN]
% HERE, Iion also englobes the bulk ion currents of the non-disc membranes 1 and 2
% and V englobes also Vi1 and Vi2

Kd=K(iDir,iDir);
Kx=K(iDir,iNeu);
Kxt=K(iNeu,iDir);
Kn=K(iNeu,iNeu);

Md=M(iDir,iDir);
Mx=M(iDir,iNeu);
Mxt=M(iNeu,iDir);
Mn=M(iNeu,iNeu);

% Compute sparse operators for Crank-Nicolson
SopLeft = (1/dt)*Mn+(1/2)*Kn; % M in uF, K in mS, operator in (mS)
SopRight= (1/dt)*Mn-(1/2)*Kn; % M in uF, K in mS, operator in (mS)
perm=symamd(SopLeft);
[~,pinv] = sort(perm);
L = chol(SopLeft(perm,perm),'lower');
Lt = L';

nsteps=frec*floor((SimDur/dt) /frec);

Ve=zeros(npoints,1);
allV=zeros(npoints+2,1); % Add 2 points for Vi1 and Vi2
allV(iDir)=vDir; 

INa1_density=zeros(npoints,1);
INa2_density=zeros(npoints,1);

% Define initial condition (e.g., resting potential)
vNeu=zeros(nNeu,1);
vNeu(end-1:end)=Vrest; % NEW: for Iclamp of the two intracellular points

% Get steady state gates
[m_ss,h_ss,j_ss,~,~,~,~,~,~] =mhjLR1_livshitz(Vrest,0); % CAREFUL! This could be the Vclamp too!

% gates and currents POINTWISE; per MEMBRANE point
%  -disc 1-----------------------------------------------------------------
m1=m_ss*ones(npoints,1);
h1=h_ss*ones(npoints,1);
j1=j_ss*ones(npoints,1);
%  -disc 2-----------------------------------------------------------------
m2=m_ss*ones(npoints,1);
h2=h_ss*ones(npoints,1);
j2=j_ss*ones(npoints,1);

% Now we also need this for the two cylinders
m_1=m_ss;
h_1=h_ss;
j_1=j_ss;

m_2=m_ss;
h_2=h_ss;
j_2=j_ss;

old_minVe=0;

% Setup parameters to export as function of time
time_arr=zeros(1,nsteps/frec);
minVe_arr=zeros(1,nsteps/frec);
maxVe_arr=zeros(1,nsteps/frec);
INa1Total_arr=zeros(1,nsteps/frec);
INa2Total_arr=zeros(1,nsteps/frec);
INa1Cyl_arr=zeros(1,nsteps/frec); 
INa2Cyl_arr=zeros(1,nsteps/frec); 
Vi1_arr=zeros(1,nsteps/frec); 
Vi2_arr=zeros(1,nsteps/frec); 

% Display Ve
h=figure(); 
set(gcf,'Position',[50 100 1800 1000])
hve=trimesh(t,p(:,1),p(:,2),p(:,3),'EdgeColor','none');
hve.FaceVertexCData=Ve;
hve.FaceColor='interp';
c=colorbar; 
c.Title.String = "Ve in mV";
title(['Time: ' num2str(time) ' ms'])
view(50,40)
axis equal
set(gca,'Clim',[-100 10])


% LOOP
for i=1:nsteps

    % Current clamp protocol
    PresentIstim1=Istim1*(time>=t_on && time<t_off);
    PresentIstim2=Istim2*(time>=t_on && time<t_off);
    
    allV(iDir)=vDir;
    allV(iNeu)=vNeu; 
    % Extract intracellular potential; either as the clamp potential or the Neumann node
    Vi1=allV(npoints+1); 
    Vi2=allV(npoints+2); 
    % Extract extracellular potential
    Ve=allV(1:npoints);
    
    % Integrate the gates (Rush-Larsen)
    %  -disc 1-------------------------------------------------------------
    % Compute membrane potential, nodewise; Vi is a scalar
    Vm1=Vi1-Ve;
    [m_inf1,h_inf1,j_inf1,edtm1,edth1,edtj1]=mhjLR1_livshitz(Vm1,dt);
    m1=m_inf1+(m1-m_inf1).*edtm1;
    h1=h_inf1+(h1-h_inf1).*edth1;
    j1=j_inf1+(j1-j_inf1).*edtj1;
    
    %  -disc 2-------------------------------------------------------------
    % Compute membrane potential, nodewise; Vi is a scalar
    Vm2=Vi2-Ve;
    [m_inf2,h_inf2,j_inf2,edtm2,edth2,edtj2]=mhjLR1_livshitz(Vm2,dt);
    m2=m_inf2+(m2-m_inf2).*edtm2;
    h2=h_inf2+(h2-h_inf2).*edth2;
    j2=j_inf2+(j2-j_inf2).*edtj2;
    
    %  -cyl 1--------------------------------------------------------------
    [m_inf_1,h_inf_1,j_inf_1,edtm_1,edth_1,edtj_1]=mhjLR1_livshitz(Vi1,dt);
    m_1=m_inf_1+(m_1-m_inf_1).*edtm_1;
    h_1=h_inf_1+(h_1-h_inf_1).*edth_1;
    j_1=j_inf_1+(j_1-j_inf_1).*edtj_1;
    
    %  -cyl 2--------------------------------------------------------------
    [m_inf_2,h_inf_2,j_inf_2,edtm_2,edth_2,edtj_2]=mhjLR1_livshitz(Vi2,dt);
    m_2=m_inf_2+(m_2-m_inf_2).*edtm_2;
    h_2=h_inf_2+(h_2-h_inf_2).*edth_2;
    j_2=j_inf_2+(j_2-j_inf_2).*edtj_2;
    
    % Compute ion currents, nodewise
    %  -disc 1-------------------------------------------------------------
    INa1=Fina1.*gNamax.*(m1.*m1.*m1.*h1.*j1).*(Vm1-ENa); % (uA)
    IK1=Fik1.*gKmax.*(Vm1-EK); % (uA)
    
    %  -disc 2-------------------------------------------------------------
    INa2=Fina2.*gNamax.*(m2.*m2.*m2.*h2.*j2).*(Vm2-ENa); % (uA)
    IK2=Fik2.*gKmax.*(Vm2-EK); % (uA)
    
    % Compute current densities
    %  -disc 1-------------------------------------------------------------
    INa1_density=INa1./F; % F = load vector unscaled for channel density (uA/cm2)
    IK1_density=IK1./F; % F = load vector unscaled for channel density (uA/cm2)
    
    %  -disc 2-------------------------------------------------------------
    INa2_density=INa2./F; % F = load vector unscaled for channel density; in uA/cm2
    IK2_density=IK2./F; % F = load vector unscaled for channel density
    
    % Sum disc currents
    Iion1=INa1+IK1; % (uA), through disc 1
    Iion2=INa2+IK2; % (uA), through disc 2
    
    % INa and IK have the same number of elements as the number of extracellular points, and they are in uA, so we can simply sum up
    INa1Total=sum(INa1); % this is the Na current for the disc 1
    INa2Total=sum(INa2); % this is the Na current for the disc 2
    IK1Total=sum(IK1); % this is the Na current for the disc 1
    IK2Total=sum(IK2); % this is the Na current for the disc 2
    % this takes into account the current at the Dirichlet points, which is still part of the Na current of the disc!
    
    % Compute currents for the cylinder membranes
    INa_1=Acyl1.*INafactorNoncleft1.*gNamax.*(m_1.*m_1.*m_1.*h_1.*j_1).*(Vi1-ENa); % (uA)
    IK_1=Acyl1.*gKmax.*(Vi1-EK); % (uA); Not scaled for any redistribution
    
    INa_2=Acyl2.*INafactorNoncleft2.*gNamax.*(m_2.*m_2.*m_2.*h_2.*j_2).*(Vi2-ENa); % (uA)
    IK_2=Acyl2.*gKmax.*(Vi2-EK); % (uA); Not scaled for any redistribution
    
    % Sum up, total ion current through the non-cleft membranes of cells 1 and 2
    Iion_1=INa_1+IK_1; % (uA)
    Iion_2=INa_2+IK_2; % (uA)
  
    %----------------------------------------------------------------------
    % To integrate, we take only the Neumann (non-Dirichlet) component of Iion
    % since the INa/IK at the Dirichlet nodes "vanishes" into ground
    Iion1_n=Iion1(iNeuDiscOnly);
    Iion2_n=Iion2(iNeuDiscOnly);
    
    SourceTerm=[Iion1_n+Iion2_n; -sum(Iion1)-Iion_1-PresentIstim1;  -sum(Iion2)-Iion_2-PresentIstim2;]; % Check sign
    % current clamp on both cells: here we need to account for the current flowing from the disc rim
    
    % compute the new vNeu
    rhs=SopRight*vNeu+SourceTerm;
    
    % Cholesky approach
    y=Lt\(L\rhs(perm));
    vNeu=y(pinv);
   
    time=time+dt;
    minVe=min(vNeu);
    maxVe=max(vNeu);
    
    if minVe<old_minVe
         old_minVe=minVe;
         VeAtMin=Ve;
         time_minVe_radius=time;
    end
    
    % Display/backup
    if mod(i,frec)==0
        
        allV(iNeu)=vNeu;
        Ve=allV(1:npoints);
        
        % Register data into time arrays
        time_arr(i/frec)=time; % (ms)
        INa1Total_arr(i/frec)=INa1Total; % (uA)
        INa2Total_arr(i/frec)=INa2Total; % (uA)
        INa1Cyl_arr(i/frec)=INa_1; % (uA)
        INa2Cyl_arr(i/frec)=INa_2; % (uA)
        minVe_arr(i/frec)=min(Ve); % (mV)
        maxVe_arr(i/frec)=max(Ve); % (mV)
        Vi1_arr(i/frec)=Vi1; % (mV)
        Vi2_arr(i/frec)=Vi2; % (mV)
            
        data=NaN(npoints,1); 
        data(1:npoints)=Ve;
           hve.FaceVertexCData=data;
           c=colorbar; 
           c.Title.String = "Ve in mV";
           title(['Time: ' num2str(time) ' ms'])
           
           drawnow
           pause(0.01)                
        
  end
    
end
% end of simulation 

%% Post processing

[pINa1,ipINa1]=min(INa1Total_arr);
[pINa2,ipINa2]=min(INa2Total_arr);
[pminVe,ipminVe]=min(minVe_arr);
tINa1=time_arr(ipINa1); 
tINa2=time_arr(ipINa2);
tminVe=time_arr(ipminVe);

% Calculate delay between action potential of cell 1 and 2
Threshold=0; 

[vVi1,ixVi1] = find(Vi1_arr(1,:)>Threshold);
[vVi2,ixVi2] = find(Vi2_arr(1,:)>Threshold);

if ~isempty(ixVi1)&& ~isempty(ixVi2)
    
% Cell 1
x11=Vi1_arr(ixVi1(1)-1);
x12=Vi1_arr(ixVi1(1));
y11=time_arr(ixVi1(1)-1);
y12=time_arr(ixVi1(1));

% Cell 2
x21=Vi2_arr(ixVi2(1)-1); 
x22=Vi2_arr(ixVi2(1));
y21=time_arr(ixVi2(1)-1);
y22=time_arr(ixVi2(1));

ActTimeCell1 =  y11+ ((y12-y11)/(x12-x11))*(Threshold-x11);
ActTimeCell2 =  y21+ ((y22-y21)/(x22-x21))*(Threshold-x21);

Vi_Delay_time=ActTimeCell2-ActTimeCell1;

else
 
Vi_Delay_time=NaN;

end

% Display results as function of time
figure() 
subplot(311) 
plot(time_arr-t_on,Vi1_arr,'k');
hold on
plot(time_arr-t_on,Vi2_arr,'--k');
xlabel('time(ms)')
ylabel('Vi (mV)')
hold off

subplot(312) % INa in discs and cyl 1 and 2
plot(time_arr-t_on,1000.*INa1Total_arr,'k'); % in nA 
hold on
plot(time_arr-t_on,1000.*INa2Total_arr,'--k'); % in nA
xlabel('time(ms)')
ylabel('INa (nA)')
hold off

subplot(313)
plot(time_arr-t_on,minVe_arr,'k'); % Ve in mV
xlabel('time(ms)')
ylabel('min Ve (mV)')

% Display in command prompt 
disp(['Minimal Ve (mV): ' num2str(min(minVe_arr))])
disp(['Delay (ms): ' num2str(Vi_Delay_time)])




