% Tortuous cardiac intercalated discs modulate ephaptic coupling
% Ena Ivanovic and Jan P. Kucera
% 2022
% Department of Physiology, University of Bern, Bern, Switzerland 

% Use under the terms of the CC-BY license (Creative Commons Attribuition 4.0 International)

close all
clear all

% Parameters to be set by the user ----------------------------------------

% Space units in cm
% Time is in ms

%Choose mesh folding pattern 
Orientation = 'Concentric'; % 'Radial' % 'Reference'
Amplitude = 4; % 2 % 4 % 8 % 16
Number = 4; % 2 % 4 % 8 % 0
filename = [Orientation 'A' num2str(Amplitude) 'N' num2str(Number) '.mat'];

% Choose Na+ channel distribution 
Config = 'uniform'; % 'cluster'  

% Variable parameters
CleftWidth = 0.03*0.0001; % in cm; Cleft width 30 nm
Vstep=-50; %-25 % Step potential (mV)
gapFactor=0;  % level of gap junctional coupling 0% of normal value 2.53 uS (Shaw & Rudy 1997) 

% -------------------------------------------------------------------------

%% Load mesh

load(filename,'pc','tc','pclust')  

% Converted Comsol values to cm, to consistently work in cm
p=0.001*pc; % 
pclust=0.001*pclust; % 
t=tc; 

% Rotate mesh in x,y axes
p=[p(:,2) p(:,3) p(:,1)];
pclust=[pclust(:,2) pclust(:,3) pclust(:,1)]; 

[ntri,~]=size(t);
[npoints,~]=size(p);

% Display ID
figure()
trimesh(t,p(:,1),p(:,2),p(:,3),'EdgeColor','k','FaceColor','[1 1 1]'); %'[0.7 0.7 0.7]'
axis equal 
grid off
axis off
% view(2)


%% Set parameters

% Constant parameter
DiscCenterX = 0;
DiscCenterY = 0;
DiscRadius = 11*0.0001;
Lcell = 100*0.0001; 
verysmall=0.1*DiscRadius;

SimDur=3;
dt=0.00005; % (ms) 50 ns
t_on=0.1;  % (ms) 
t_off=3.0; % (ms)
frec=100;  % downsampling

Cm=1; % Specific membrane capacitance (uF/cm2) 
SigmaExt=1000*(1/150); % Conductivity 150 Ohm.cm = 10 mS/cm
specific_gj_resistance=1.5; % Ohm.cm2 (Luo and Rudy, 1994)

gNamax=23; % maximal conductance (mS/cm2) 
gKmax=0.3; % maximal conductance (mS/cm2) 

ENa=55; % Nernst potential Na (mV)
EK=-85; % Nernst potential K (mV)
Vhold=EK; % Holding potential (mV)
Vrest=EK; % Resting potential (mV)

% Calculate areas
Alat=2*pi*DiscRadius*Lcell; % Area lateral membrane (cm2) 
Ajun= pi*(DiscRadius^2); % Area flat ID membrane (cm2) 
Acyl1=Alat+Ajun; % Cell 1 (cm2) 
Acyl2=Alat+Ajun; % Cell 2 (cm2) 
Aflat=Alat+2*Ajun; % Area cell membrane with only flat IDs (cm2) 

[~,~,~,Ac,~]=f_kmf_lintric_inh_3D(p',t',1,1,1); 
AID = sum(Ac); % Area (tortuous) ID membrane (cm2) 
SF_ID =Ajun/AID; % Tortuosity factor (unitless)
Atort = Alat + Ajun + AID; % Total cell membrane area with tortuous ID (cm2)  

% Distribition Na+ channels in cell membrane (Hichri et al. 2018; Ivanovic & Kucera 2021)
INafactor=5.05; % (unitless)
INafactorNoncleft1=0.555; % (unitless)
INafactorNoncleft2=0.555; % (unitless)

% Gap junctional coupling
specific_gj_conductance=1/specific_gj_resistance; % (S/cm2)
G=1000*gapFactor*(Ajun/2)*specific_gj_conductance; % Total intercellular conductance (mS) 

% Find triangles for Na+ channel cluster
switch (Config)
    
    case 'cluster'
        
boole=zeros(ntri,3);
for i=1:ntri
   for j=1:3 % 3 corners per triangle
        ix=t(i,j);
        b=ismember(p(ix,:),pclust,'rows');              
        boole(i,j)=b;
   end
end
% Indices of boundary triangles that are on the inner surface of pe domain
ixc=find(sum(boole,2)==3);
ttclust=t(ixc,:); % inner surface triangles cell2
figure()
trimesh(t,p(:,1),p(:,2),p(:,3),'facecolor','[0.7 0.7 0.7]','edgecolor','k');
hold on
trimesh(ttclust,p(:,1),p(:,2),p(:,3),'facecolor','r','edgecolor','k');
hold off
axis equal

[~,~,~,AcNa,~]=f_kmf_lintric_inh_3D(p',ttclust',1,1,1);
ANa = sum (AcNa); 
ClusterFactorNa =AID/ANa; 

end

%% ------------------------------------------------------------------------

kk=SigmaExt.*CleftWidth; % cm * mS/cm -> mS; conductance of a "square" of extracellular space
mm=Cm; % (uF/cm2)
ff=1; % (unitless)

% Scaling conductance TRIANGLEWISE, Computation of V, Iion and gates NODEWISE
[K,M,F,A,X]=f_kmf_lintric_inh_3D(p',t',kk,mm,ff); % kk, mm and ff are either triangle-wise or scalar
% kk will always remain scalar unless one incorporates heterogeneities in cleft width or extracellular resistance
% K is in mS, laplacian operator incorporating, if needed, heterogeneities of clw as different entries in kk
% M is in uF, capacitive mass matrix
% F is in area units (since ff=1)
% A is the area (cm2) of the triangles
% Here, K and F are the most important, the others will not be used

% Since we assume that Cm is homogeneous and the same for both membranes;
% no need to differentiate between M for the two cells


%% Ionic currents parameters

%========================= cell1 : Driver cell ============================
IK_scaling_factor1=ones(ntri,1); % unitless

switch (Config)
       case 'uniform'
INa_scaling_factor1=ones(ntri,1).*INafactor; % unitless
       case 'cluster'
INa_scaling_factor1=zeros(ntri,1); % unitless
INa_scaling_factor1(ixc)=INafactor*ClusterFactorNa;
end

% Compute scaled load vector 
[~,~,Fina1,~,~]=f_kmf_lintric_inh_3D(p',t',1,INa_scaling_factor1,INa_scaling_factor1); % for INa in cell 1
[~,~,Fik1,~,~]=f_kmf_lintric_inh_3D(p',t',1,IK_scaling_factor1,IK_scaling_factor1); % for IK in cell 1
% Fina1 and Fik1: unit: area, since INa_scaling_factor1 is unitless (normalized)

Fina1=Fina1.*SF_ID; 
Fik1=Fik1.*SF_ID; 

gNa1=sum(gNamax.*Fina1); 
gK1=sum(gKmax.*Fik1); 

%========================= cell2 : Follower cell ==========================
IK_scaling_factor2=ones(ntri,1); % unitless

switch (Config)
       case 'uniform'
INa_scaling_factor2=ones(ntri,1).*INafactor; % unitless
       case 'cluster'
INa_scaling_factor2=zeros(ntri,1); % unitless
INa_scaling_factor2(ixc)=INafactor*ClusterFactorNa;
end

% Compute scaled load vector
[~,~,Fina2,~,~]=f_kmf_lintric_inh_3D(p',t',1,INa_scaling_factor2,INa_scaling_factor2); % for INa in cell 2
[~,~,Fik2,~,~]=f_kmf_lintric_inh_3D(p',t',1,IK_scaling_factor2,IK_scaling_factor2); % for IK in cell 2
% Fina2 and Fik2: unit: area, since INa_scaling_factor2 is unitless (normalized)

Fina2=Fina2.*SF_ID; 
Fik2=Fik2.*SF_ID; 

gNa2=sum(gNamax.*Fina2); 
gK2=sum(gKmax.*Fik2); 

%--------------------------------------------------------------------------
%Calculate vector m columnwise as sum of the rows of M

mvector=sum(M,2); % sparse
mscalar=full(sum(mvector)); % Calculate scalar mscalar as sum of mvector


%% Boundary conditions

dFromCenter = (p(:,1)-DiscCenterX).^2 + (p(:,2)-DiscCenterY).^2;
dc = ( abs(dFromCenter-(DiscRadius^2))<verysmall^2 ); 

iDir=find(dc==1); % indices of points with Dirichlet condition
iNeu=find(dc~=1); % indices of points with non Dirichlet condition (Neumann)
nDir=length(iDir);
nNeu=length(iNeu);
vDir=zeros(nDir,1); % vector of potentials at the dc points, here all 0

% Display boundary conditions
figure()
trimesh(t,p(:,1),p(:,2),p(:,3),'EdgeColor','k','FaceColor','[0.7 0.7 0.7]');
hold on 
plot3(p(iDir,1),p(iDir,2),p(iDir,3),'xb')
hold off 
axis equal

%% Figure 3

r1=0; 
r2=0.25*DiscRadius; 
r3=0.5*DiscRadius; 
r4=0.75*DiscRadius; 
r5=DiscRadius;

verysmall1=0.001*DiscRadius;
verysmall2=0.15*DiscRadius;

dc1 = ( abs(dFromCenter-(r1^2))<verysmall1^2 ); 
dc2 = ( abs(dFromCenter-(r2^2))<verysmall2^2 ); 
dc3 = ( abs(dFromCenter-(r3^2))<verysmall2^2 ); 
dc4 = ( abs(dFromCenter-(r4^2))<verysmall2^2 ); 
dc5 = dc; 

ir1=find(dc1==1);
ir2=find(dc2==1);
ir3=find(dc3==1); 
ir4=find(dc4==1); 
ir5=find(dc5==1); 

%% Simulation

time=0; % ms

% The system is K*Ve = M*dVe/dt + Iion % unit: uA = + (uA/uF)*uF + uA
% under Dirichlet condition
% Iion MUST be in (uA)

% Split as
% [Kd  Kx][Vd]=[Md  Mx][dVd/dt]+[iIonD]
% [Kxt Kn][Vn]=[Mxt Mn][dVn/dt]+[iIonN]
% HERE, Iion also englobes the bulk ion currents of the non-disc membranes 1 and 2
% and V englobes also Vi1 and Vi2

Kd=K(iDir,iDir);
Kx=K(iDir,iNeu);
Kxt=K(iNeu,iDir);
Kn=K(iNeu,iNeu);

Md=M(iDir,iDir);
Mx=M(iDir,iNeu);
Mxt=M(iNeu,iDir);
Mn=M(iNeu,iNeu);

% We assume from here that Vd=0 and dVd/dt is 0
% Kn*Vn = Mn*dVn/dt - Kxt*Vd + iIonN; % can be solved by Crank-Nicolson using Kn and Mn
% Moreover, if VDir=0, the system simplifies to
% Kn*Vn = Mn*dVn/dt + iIonN;

% Compute sparse operators for Crank-Nicolson
SopLeft = 2*(1/dt)*Mn+(1/2)*Kn; % M in uF, K in mS, operator in (mS)
SopRight= 2*(1/dt)*Mn-(1/2)*Kn; % M in uF, K in mS, operator in (mS)
perm=symamd(SopLeft);
[~,pinv] = sort(perm);
L = chol(SopLeft(perm,perm),'lower');
Lt = L';

nsteps=frec*floor((SimDur/dt) /frec);

% Define initial condition (e.g., resting potential)
Ve=zeros(npoints,1);
vNeu=zeros(nNeu,1);

INa1_density=zeros(npoints,1);
INa2_density=zeros(npoints,1);

% Get steady state gates
[m_ss,h_ss,j_ss,~,~,~,~,~,~] =mhjLR1_livshitz(Vrest,0); 

% Gates and currents POINTWISE; per MEMBRANE point
%  -disc 1-----------------------------------------------------------------
m1=m_ss*ones(npoints,1);
h1=h_ss*ones(npoints,1);
j1=j_ss*ones(npoints,1);
%  -disc 2-----------------------------------------------------------------
m2=m_ss*ones(npoints,1);
h2=h_ss*ones(npoints,1);
j2=j_ss*ones(npoints,1);

% Now we also need this for the two cylinders 
m_1=m_ss;
h_1=h_ss;
j_1=j_ss;

m_2=m_ss;
h_2=h_ss;
j_2=j_ss;

old_minVe=0;

% Setup parameters to export as function of time
time_arr=zeros(1,nsteps/frec);
minVe_arr=zeros(1,nsteps/frec);
maxVe_arr=zeros(1,nsteps/frec);
INa1Total_arr=zeros(1,nsteps/frec);
INa2Total_arr=zeros(1,nsteps/frec);
INa1Cyl_arr=zeros(1,nsteps/frec); 
INa2Cyl_arr=zeros(1,nsteps/frec); 
Vi1_arr=zeros(1,nsteps/frec); 
Vi2_arr=zeros(1,nsteps/frec); 
INa1_density_arr1=zeros(1,nsteps/frec); 
INa2_density_arr1=zeros(1,nsteps/frec); 
INa1_density_arr2=zeros(1,nsteps/frec); 
INa2_density_arr2=zeros(1,nsteps/frec); 
INa1_density_arr3=zeros(1,nsteps/frec); 
INa2_density_arr3=zeros(1,nsteps/frec); 
INa1_density_arr4=zeros(1,nsteps/frec); 
INa2_density_arr4=zeros(1,nsteps/frec); 
INa1_density_arr5=zeros(1,nsteps/frec); 
INa2_density_arr5=zeros(1,nsteps/frec); 
Ve_arr2=zeros(1,nsteps/frec);  % (mV)
Ve_arr3=zeros(1,nsteps/frec);  % (mV)
Ve_arr4=zeros(1,nsteps/frec);  % (mV)
Ve_arr5=zeros(1,nsteps/frec);  % (mV)

% Display Ve
h=figure(); 
set(gcf,'Position',[50 100 1800 1000])
hve=trimesh(t,p(:,1),p(:,2),p(:,3),'EdgeColor','none');
hve.FaceVertexCData=Ve;
hve.FaceColor='interp';
c=colorbar; 
c.Title.String = "Ve in mV";
title(['Time: ' num2str(time) ' ms'])
view(50,40)
axis equal
set(gca,'Clim',[-100 10])


% LOOP
for i=1:nsteps

    % Voltage clamp protocol
    Vi1=(Vhold * (time<t_on)) + (Vstep * (time>=t_on && time<t_off)) + (Vhold * (time>t_off));
    Vi2=Vhold;
    
    % Construct Ve
    Ve(iDir)=vDir;
    Ve(iNeu)=vNeu;
               
    % Integrate the gates (Rush-Larsen)
    %  -disc 1-------------------------------------------------------------
    % Compute membrane potential (nodewise); Vi is a scalar
    Vm1=Vi1-Ve;
    [m_inf1,h_inf1,j_inf1,edtm1,edth1,edtj1]=mhjLR1_livshitz(Vm1,dt);
    m1=m_inf1+(m1-m_inf1).*edtm1;
    h1=h_inf1+(h1-h_inf1).*edth1;
    j1=j_inf1+(j1-j_inf1).*edtj1;
    
    %  -disc 2-------------------------------------------------------------
    % Compute membrane potential (nodewise); Vi is a scalar
    Vm2=Vi2-Ve;
    [m_inf2,h_inf2,j_inf2,edtm2,edth2,edtj2]=mhjLR1_livshitz(Vm2,dt);
    m2=m_inf2+(m2-m_inf2).*edtm2;
    h2=h_inf2+(h2-h_inf2).*edth2;
    j2=j_inf2+(j2-j_inf2).*edtj2;
    
    %  -cyl 1--------------------------------------------------------------
    [m_inf_1,h_inf_1,j_inf_1,edtm_1,edth_1,edtj_1]=mhjLR1_livshitz(Vi1,dt);
    m_1=m_inf_1+(m_1-m_inf_1).*edtm_1;
    h_1=h_inf_1+(h_1-h_inf_1).*edth_1;
    j_1=j_inf_1+(j_1-j_inf_1).*edtj_1;
    
    %  -cyl 2--------------------------------------------------------------
    [m_inf_2,h_inf_2,j_inf_2,edtm_2,edth_2,edtj_2]=mhjLR1_livshitz(Vi2,dt);
    m_2=m_inf_2+(m_2-m_inf_2).*edtm_2;
    h_2=h_inf_2+(h_2-h_inf_2).*edth_2;
    j_2=j_inf_2+(j_2-j_inf_2).*edtj_2;
    
    % Compute ion currents, nodewise
    %  -disc 1-------------------------------------------------------------
    INa1=Fina1.*gNamax.*(m1.*m1.*m1.*h1.*j1).*(Vm1-ENa); % (uA)
    IK1=Fik1.*gKmax.*(Vm1-EK); % (uA)
    
    %  -disc 2-------------------------------------------------------------
    INa2=Fina2.*gNamax.*(m2.*m2.*m2.*h2.*j2).*(Vm2-ENa); % (uA)
    IK2=Fik2.*gKmax.*(Vm2-EK); % (uA)
    
    % Compute current densities
    %  -disc 1-------------------------------------------------------------
    INa1_density=INa1./F; % F = load vector unscaled for channel density (uA/cm2)
    IK1_density=IK1./F; % F = load vector unscaled for channel density (uA/cm2)
    
    %  -disc 2-------------------------------------------------------------
    INa2_density=INa2./F; % F = load vector unscaled for channel density; (uA/cm2)
    IK2_density=IK2./F; % F = load vector unscaled for channel density
    
    % Sum disc currents
    Iion1=INa1+IK1; % (uA), through disc 1
    Iion2=INa2+IK2; % (uA), through disc 2
    
    % INa and IK have the same number of elements as the number of extracellular points, and they are in uA, so we can simply sum up
    INa1Total=sum(INa1); % this is the Na current for the disc 1
    INa2Total=sum(INa2); % this is the Na current for the disc 2
    IK1Total=sum(IK1); % this is the K current for the disc 1
    IK2Total=sum(IK2); % this is the K current for the disc 2
    % this takes into account the current at the Dirichlet points, which are still part of the Na current of the disc!
    
    % Compute currents for the cylinder membranes
    INa_1=Acyl1.*INafactorNoncleft1.*gNamax.*(m_1.*m_1.*m_1.*h_1.*j_1).*(Vi1-ENa); % (uA)
    IK_1=Acyl1.*gKmax.*(Vi1-EK); % (uA); Not scaled for any redistribution
    
    INa_2=Acyl2.*INafactorNoncleft2.*gNamax.*(m_2.*m_2.*m_2.*h_2.*j_2).*(Vi2-ENa); % (uA)
    IK_2=Acyl2.*gKmax.*(Vi2-EK); %(uA); Not scaled for any redistribution
    
    % Sum up, total ion current through the non-cleft membranes of cells 1 and 2
    Iion_1=INa_1+IK_1; % (uA)
    Iion_2=INa_2+IK_2; % (uA)
  
    %----------------------------------------------------------------------
    % To integrate, we take only the Neumann (non-Dirichlet) component of Iion
    Iion1_n=Iion1(iNeu);
    Iion2_n=Iion2(iNeu);
    
    SourceTerm=Iion1_n+Iion2_n; 
    
    rhs=SopRight*vNeu+SourceTerm;
    
    % Cholesky approach
    y=Lt\(L\rhs(perm));
    vNeu=y(pinv);

    time=time+dt;
    minVe=min(vNeu);
    
    if minVe<old_minVe
         old_minVe=minVe;
         VeAtMin=Ve;
         time_minVe_radius=time;
    end
    
    % Display/backup
    if mod(i,frec)==0
        
        % Register data into time arrays
        time_arr(i/frec)=time; % (ms)
        INa1Total_arr(i/frec)=INa1Total; % (uA)
        INa2Total_arr(i/frec)=INa2Total; % (uA)
        INa1Cyl_arr(i/frec)=INa_1; % (uA)
        INa2Cyl_arr(i/frec)=INa_2; % (uA)
        minVe_arr(i/frec)=min(Ve); % (mV)
        maxVe_arr(i/frec)=max(Ve); % (mV)
        Vi1_arr(i/frec)=Vi1; % (mV)
        Vi2_arr(i/frec)=Vi2; % (mV)  
        Ve(iNeu)=vNeu; % (mV)
        INa1_density_arr1(i/frec)=mean(INa1_density(ir1)); % (uA/cm2)
        INa2_density_arr1(i/frec)=mean(INa2_density(ir1)); % (uA/cm2)
        INa1_density_arr2(i/frec)=mean(INa1_density(ir2)); % (uA/cm2)
        INa2_density_arr2(i/frec)=mean(INa2_density(ir2)); % (uA/cm2)
        INa1_density_arr3(i/frec)=mean(INa1_density(ir3)); % (uA/cm2)
        INa2_density_arr3(i/frec)=mean(INa2_density(ir3)); % (uA/cm2)
        INa1_density_arr4(i/frec)=mean(INa1_density(ir4)); % (uA/cm2)
        INa2_density_arr4(i/frec)=mean(INa2_density(ir4)); % (uA/cm2)
        INa1_density_arr5(i/frec)=mean(INa1_density(ir5)); % (uA/cm2)
        INa2_density_arr5(i/frec)=mean(INa2_density(ir5)); % (uA/cm2)
        Ve_arr1(i/frec)=min(Ve); % (mV)
        Ve_arr2(i/frec)=mean(Ve(ir2)); % (mV)
        Ve_arr3(i/frec)=mean(Ve(ir3)); % (mV)
        Ve_arr4(i/frec)=mean(Ve(ir4)); % (mV)
        Ve_arr5(i/frec)=mean(Ve(ir5)); % (mV)
            
        data=NaN(npoints,1); 
        data(1:npoints)=Ve;
           hve.FaceVertexCData=data;
           c=colorbar; 
           c.Title.String = "Ve in mV";
           title(['Time: ' num2str(time) ' ms'])
            
           drawnow
           pause(0.01)                 
 end
    
end
% end of simulation 

%% Post processing

% Figure 3
figure()

subplot(311)
plot(time_arr-t_on,1000.*INa1Total_arr,'k'); % (nA)
hold on 
plot(time_arr-t_on,1000.*INa2Total_arr,'--k'); % (nA)
hold off
xlim([0 3])
ylim([-10 0])
title('INa ID (nA)')

subplot(312)
plot(time_arr-t_on,INa1_density_arr1,'k'); 
hold on 
plot(time_arr-t_on,INa2_density_arr1,'--k'); 
plot(time_arr-t_on,INa1_density_arr2,'g'); 
plot(time_arr-t_on,INa2_density_arr2,'--g'); 
plot(time_arr-t_on,INa1_density_arr3,'b'); 
plot(time_arr-t_on,INa2_density_arr3,'--b'); 
plot(time_arr-t_on,INa1_density_arr4,'r'); 
plot(time_arr-t_on,INa2_density_arr4,'--r'); 
hold off
xlim([0 3])
ylim([-1500 0])
title('INa density ID (uA/cm2)')

subplot(313)
plot(time_arr-t_on,minVe_arr,'k'); % (mV)
hold on 
plot(time_arr-t_on,Ve_arr2,'g'); % (mV)
plot(time_arr-t_on,Ve_arr3,'b'); % (mV)
plot(time_arr-t_on,Ve_arr4,'r'); % (mV)
hold off
xlim([0 3])
ylim([-100 0])
title('Ve (mV)')
legend('r=0','r=0.25 R','r=0.5 R','r=0.75 R')


 
