function [K,M,F,A,X]=f_kmf_lintric_inh_3D(p,t,kk,mm,ff)

% Computes the stiffness matrix K and the mass matrix M (in sparse format) 
% as well as the load vector F and the area A (non sparse) for given
% triangles embedded in 3 dimensional space

% Matlab functions called:
% kron (Matlab function)
% sparse (Matlab sparse matrix)

% Custom functions called:
% f_kmf_lintricontrib
% fCreateUs

% Inputs
% kk,mm,ff : either arrays of coefficients corresponding to the triangles, or scalars
% p,t points and triangles as returned by MATLAB's "generateMesh" or another meshing program (converted to Matlab format)

% Outputs
% K stiffness matrix (matrix of conductances) in Matlab sparse format, npoints by npoints
% M Mass matrix (matrix of capacitances) 
% F Load vector, unscaled column vector of "npoints" elements (not sparse)
% A 1D array areas of the triangle areas
% X not used 

% for debugging purposes
% kk=1;
% mm=1;
% ff=1;

% "c" in lintri"c" stands for "combo"

% What does the function do?

% Given triangles t (n,3) matrix in the same uniform isotropic domain d (the
% 4th row of t), compute corresponding K and M (in sparse format) as well as F and A (non sparse)

% Same convention as Matlab for p and t

% Not scaled for c a and f; do it yourself afterwards

szp=size(p);
npoints=szp(2); % number of points

szt=size(t);
ntri=szt(2); % number of triangles
% Each triangle contributes a 3x3 subset to A and M and 3x1 to F

% Allocate A and F
A=zeros(1,ntri); % the areas
F=zeros(npoints,1); % that vector is rarely sparse

% Don't allocate K and M, because these will be sparse
% K=zeros(npoints,npoints); % will be computed in sparse format at the end
% M=zeros(npoints,npoints); % will be computed in sparse format at the end

% Create triangle-point map X (anyway, this used anymore, these lines can be commented
jxt=t(1:3,:); % Take the indices of the triangle corners, disregard the domain
jxt=jxt(:); % Concatenate to one long series of triplets (becomes a column array)
ixt=kron(1:ntri,[1 1 1]); % 1 1 1 2 2 2 3 3 3 etc...
X=sparse(ixt,jxt,ones(1,3*ntri),ntri,npoints);
% element in row R and column C are 1 if triangle R contains point C
% nonzero elements in a row are the indices of the corners of a given triangle
% nonzero elements in a column are the triangles to which a given point belong 

% Convert to arrays if passed as scalars
if length(kk)==1
    kk=repmat(kk,1,ntri);
end

if length(mm)==1
    mm=repmat(mm,1,ntri);
end

if length(ff)==1
    ff=repmat(ff,1,ntri);
end

% error check
if length(kk)~=ntri || length(mm)~=ntri || length(ff)~=ntri
    disp('kk/mm/ff must have as many elements as triangles')
end

% Allocate arrays to be used by fCreateUs
kf=zeros(1,3*ntri); % indices of origin nodes (from)
kt=zeros(1,3*ntri); % indices of destination nodes (to)
kv=zeros(1,3*ntri); % corresponding values in K (will be summed)
mv=zeros(1,3*ntri); % corresponding values in M (will be summed)
    
ctr=0; % counter, will be incremented by three for every passage of the loop

% LOOP through the triangles

for i=1:ntri
    
    % i is now the index of the triangle
    i1=t(1,i); % index of corner 1
    i2=t(2,i); % index of corner 2
    i3=t(3,i); % index of corner 3

    % compute contributions k m f a of that triangle to K M F A 
    [k,m,f,a]=f_kmf_lintricontrib_3D(p(:,i1),p(:,i2),p(:,i3),1);
    % k and m are 3x3 matrices
    % f is a vector of 3 elements
    % a is scalar (area)
        
    % Scale with kk, mm, ff 
    k=kk(i)*k;
    m=mm(i)*m;
    f=ff(i)*f;
    % a obviously not scaled
        
    % To optimize speed, this function could be expanded inline with
    % computation only of 3 instead of 9 values
   
    % Add contribution to F
    F([i1 i2 i3])=F([i1 i2 i3])+f;
    % Store in A
    A(i)=a;

    % This is what we want in the end:
    % K([i1 i2 i3],[i1 i2 i3])=K([i1 i2 i3],[i1 i2 i3])+k; 
    % M([i1 i2 i3],[i1 i2 i3])=M([i1 i2 i3],[i1 i2 i3])+m; % computed at the end in sparse form
    % but to optimize memory management and speed, rather than writing into matrix 
    % at every passage of the loop, these data are stored in kf, kt, kv and mv    
    % and only at the end K and M are computed in sparse form
    
    % Prepare for sparse computation and store into kf, kt, kv and mv
    % Connecting i1 to i2 with k(1,2)
    % Connecting i2 to i3 with k(2,3)
    % Connecting i3 to i1 with k(1,3) 
    kf(ctr+1:ctr+3)=[i1 i1 i2];
    kt(ctr+1:ctr+3)=[i2 i3 i3];
    kv(ctr+1:ctr+3)=[k(1,2) k(1,3) k(2,3)];         
    mv(ctr+1:ctr+3)=[m(1,2) m(1,3) m(2,3)];

    ctr=ctr+3; % increment counter by three

end
    
% Generate the sparse matrices

% obtain incidence matrix of the directed graph
U=fCreateUs(npoints,kf,kt); % obtain incidence matrix of the directed graph, with -1s and +1s, else 0s
Ua=abs(U); % incidence matrix of the undirected graph (only 0s and 1s

S=sparse(1:length(kv),1:length(kv),-kv); % diagonal matrix; - sigh to have Matlab positive semidefinite convention
% the following algebra connects everything correctly
% sums where needed (node connected to many others)
% and diagonal is such that sum of columns / rows is 0
K=U*S*U'; 
% K is symmetric and positive semidefinite, determinant is 0
% diagonal positive
% rest 0 or negative

S=sparse(1:length(mv),1:length(mv),mv); % diagonal matrix
% the following algebra also connects everything correctly
% sums where needed (node connected to many others)
M=Ua*S*Ua';
% M is symmetric, positive definite
% the total sum is the total area if the weights mm 8densities) are 1
% all nonzero elements are positive (no negative mass)


%[Kt,Mt,Ft] = assema(p,t,1,1,1); % this is the corresponding Matlab function...

