function [k,m,f,a]=f_kmf_lintricontrib_3D(p1,p2,p3,sigma)

% Compute finite element contribution of triangle p1 p2 p3
% to k (stiffness), m (mass), f (load) and a (area)

% Input: p1, p2, p3, coordinates of the corners (2-element vectors)
% sigma: diffusion coefficient (scalar) or 2x2 conductivity tensor
% sigma corresponds to the conductance of a square of intercalated disc extracellular space

% Output:
% contribution k (stiffness)
% contribution m (mass)
% contribution f (load)
% a (area)

% to debug/test
% p1=[0 0 0];
% p2=[0 1 0];
% p3=[1 0 0];
% sigma=1;

% assume linear triangles and uniform scalar diffusion (given by sigma) within the triangle 
% assume uniform load and density of 1 within the triangle (scale afterwards if you wish)

x1=p3(1)-p2(1); % x component of vector between corners 2 and 3
y1=p3(2)-p2(2); % y component of vector between corners 2 and 3
z1=p3(3)-p2(3); % z component of vector between corners 2 and 3

x2=p1(1)-p3(1); % x component of vector between corners 3 and 1
y2=p1(2)-p3(2); % y component of vector between corners 3 and 1
z2=p1(3)-p3(3); % z component of vector between corners 3 and 1

x3=p2(1)-p1(1); % x component of vector between corners 1 and 2
y3=p2(2)-p1(2); % y component of vector between corners 1 and 2
z3=p2(3)-p1(3); % z component of vector between corners 1 and 2

% Compute area
%a=abs(x1*y2-x2*y1)/2 % classical formula based on the cross product/determinant 
a=sqrt((y1*z2-y2*z1)^2+(z1*x2-z2*x1)^2+(x1*y2-x2*y1)^2)/2; % take into account the third dimension

% mass contribution
% can be computed by explicit integration, see Matlab PDE documentation
m=(eye(3)+1)*(a/12); 

% load contribution
% can be computed by explicit integration, see Matlab PDE documentation
f=ones(3,1)*(a/3); % See Matlab PDE documentation

% stiffness contribution
% can be computed by explicit integration, see Matlab PDE documentation
%v=[[y1 -x1]; [y2 -x2]; [y3 -x3];]; % standard
%v=[[y1 x1]; [y2 x2]; [y3 x3];]; % works too, sice elements are squared
%v=[[x1 y1]; [x2 y2]; [x3 y3];]% this would be for 2D
v=[[x1 y1 z1]; [x2 y2 z2]; [x3 y3 z3];]; % take third dimension "z" into account
k=(1/(4*a)).*(v*sigma*v'); % sigma can be tensor or scalar

% Note that k is symmetric positive semidefinite


